package core;

import integration.League;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;

import util.ExchangerInstance;
import util.Exchangers;

import com.vvk.better.exceptions.LoadingConfDataException;
import com.vvk.better.models.Exchanger;
import com.vvk.better.models.Match;
import com.vvk.better.models.Site;

public class MatchesMapper extends Mapper<LongWritable, Text, Text, Text> {

	private static final String EXCHANGER_NAME 	= "exchanger";
	private static final String LEAGUE_NAME 	= "league_name";
	private static final String LEAGUE_COUNTRY 	= "country";
	
	public static final Log LOG = LogFactory.getLog(MatchesMapper.class);

	public void map(LongWritable key, Text value, Context context) throws IOException, InterruptedException {
		//value looks like <exchanger_name>name</exchanger_name><league_name>league</league_name><league_country>country</league_country>
		List<Match> matches = getMatches(value.toString());
		
//		if(matches.size() == 0) {
//			throw new RuntimeException("No mathces found!");
//		}
		
		Text outputKey = new Text();
		Text outputValue = new Text();

		for(Match match : matches){
			String matchInfo = match.getMatchInfo();
			//looks like [Team1 Name] [Team2 Name] [1 - 1.4/3000] [X - 3.2/2000] [2 - 1.8/1000]
			Pattern pattern = Pattern.compile("\\[[a-zA-Z0-9 \\.,-/]+\\]");
			Matcher mathcer = pattern.matcher(matchInfo);
			
			ArrayList<String> parsedItems = new ArrayList<String>();
			while (mathcer.find()){
				parsedItems.add(matchInfo.substring(mathcer.start(), mathcer.end()));
			}
			
			outputKey.set(parsedItems.get(0) + parsedItems.get(1));
			outputValue.set(parsedItems.get(2) + " " + parsedItems.get(3) + " " + parsedItems.get(4));
			context.write( outputKey, outputValue );

			outputKey.clear();
			outputValue.clear();
		}
	}

	public List<Match> getMatches(String line) throws IOException {
		
		Exchanger instance;
		try {
			instance = extractExchanger(line);
		} catch (LoadingConfDataException e) {
			throw new IOException(e);
		}
		League league = extractLeague(line);
		
		if(instance == null || league == null) {
			return new ArrayList<Match>();
		}
		Site site = new Site(instance, league);
		return site.getParsedData();
	}

	private League extractLeague(String line) {
		int leagueStartIndex = line.indexOf("<" + LEAGUE_NAME + ">");
		if(leagueStartIndex < 0) {
			throw new RuntimeException("No league found!");
		}
		leagueStartIndex += LEAGUE_NAME.length() + 2;
		int leagueEndIndex = line.indexOf("</" + LEAGUE_NAME + ">");
		String leagueName = line.substring(leagueStartIndex, leagueEndIndex);
		String leagueCountry = line.substring(line.indexOf("<" + LEAGUE_COUNTRY + ">") + LEAGUE_COUNTRY.length() + 2,
				  line.indexOf("</" + LEAGUE_COUNTRY + ">"));
		League league = new League(leagueName, leagueCountry);
		return league;
	}

	private Exchanger extractExchanger(String line) throws LoadingConfDataException {
		int exchangerStartIndex = line.indexOf("<" + EXCHANGER_NAME + ">");
		if(exchangerStartIndex < 0) {
			throw new RuntimeException("No exchanger found!");
		}
		exchangerStartIndex += EXCHANGER_NAME.length() + 2;
		int exchangerEndIndex = line.indexOf("</" + EXCHANGER_NAME + ">");
		String exchangerName = line.substring(exchangerStartIndex, exchangerEndIndex);
		ExchangerInstance exchanger = Exchangers.valueOf(exchangerName);
		Exchanger instance = exchanger.getExchangerInstance();
		instance.loadConfiguration();
		return instance;
	}

}
