package core;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;

import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Reducer;

public class VolumeCounterPerCoefficient extends Reducer<Text, Text, Text, Text> {
	
	public void reduce(Text key, Iterable<Text> values, Context context) throws IOException, InterruptedException {

		BigDecimal[] aggregatedVolumePerCoefficient = {new BigDecimal("0"), new BigDecimal("0"), new BigDecimal("0")};
		BigDecimal[] coefficients = {new BigDecimal("1"), new BigDecimal("1"), new BigDecimal("1")};;
		
		int numberOfData = 0;
		for (Text markets : values) {
			String[] marketsValues = markets.toString().split(" ");
			
			for(String singleMarket : marketsValues) {
				String market[] = singleMarket.split("-");
				
				String marketKey = market[0].substring(1);
				String volCoef = market[1].substring(0, market[1].length()-1); 
				String marketValues[] = volCoef.split("/");
			
				BigDecimal volume = new BigDecimal(marketValues[1].trim());
				BigDecimal coefficient = new BigDecimal(marketValues[0].trim());
				BigDecimal volumeForMarketKey = volume.divide(coefficient, 2, RoundingMode.HALF_UP);
				int marketId;
				if(marketKey.equals("X")){
					marketId = 1;
				} else {
					marketId = Integer.parseInt(marketKey) + 1;
				}
				aggregatedVolumePerCoefficient[marketId - 1] = aggregatedVolumePerCoefficient[marketId - 1].add(volumeForMarketKey);
				coefficients[marketId - 1] = coefficients[marketId - 1].add(coefficient);
			}
			numberOfData++;
		}
		
		for(int i=0; i<2; i++){
			aggregatedVolumePerCoefficient[i] = aggregatedVolumePerCoefficient[i].divide(BigDecimal.valueOf(numberOfData),  2, RoundingMode.HALF_UP);
			coefficients[i] = coefficients[i].divide(BigDecimal.valueOf(numberOfData),  2, RoundingMode.HALF_UP);
		}
		
		StringBuilder newValue = new StringBuilder();
		newValue.append("[1 - ");
		newValue.append(coefficients[1]);
		newValue.append("/" + aggregatedVolumePerCoefficient[1] +"] ");
		
		newValue.append("[X - ");
		newValue.append(coefficients[0]);
		newValue.append("/" + aggregatedVolumePerCoefficient[0] +"] ");

		newValue.append("[2 - ");
		newValue.append(coefficients[2]);
		newValue.append("/" + aggregatedVolumePerCoefficient[2] +"] ");
		
		Text word = new Text();
		word.set(newValue.toString());
		context.write(key, word);
		word.clear();
	}

}
