package integration;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Map;
import java.util.TreeMap;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;

import com.vvk.better.util.CommonConstants;

public class League extends com.vvk.better.models.League{

	public League(String name, String country) {
		super(name, country);
		Map<String, String> teamNames = loadTeamNames();
		if(teamNames.size() == 0) {
			throw new RuntimeException("No names for league: " + name + " " + country);
		}
		setTeamNames(teamNames);
	}
	
	@Override
	protected Map<String, String> loadTeamNames() {
		
		
		TreeMap<String, String> map = new TreeMap<String, String>();
		String fileName = getName() + "_" + getCountry() + ".txt";
		Path path = new Path(CommonConstants.RESOURCE_PATH + File.separatorChar + fileName);
		BufferedReader reader = null;
		try
		{
			FileSystem hdfs = FileSystem.get(new Configuration());
			FSDataInputStream stream = hdfs.open(path);
			reader = new BufferedReader(new InputStreamReader(stream));
			String line;
			while ((line = reader.readLine()) != null && !line.equals(""))
			{
				String[] mappers = line.split(",");
				map.put(mappers[1].trim().toLowerCase(), mappers[0].trim());
			}
		}
		catch (FileNotFoundException e)
		{
			throw new RuntimeException(e);
//			log.warn("Warning - no names provided for league - " + name + "!");
		}
		catch (Exception ex)
		{
			throw new RuntimeException(ex);
//			log.error("Problems during loading team names!", ex);
		}
		finally
		{
			try
			{
				if (reader != null) reader.close();
			}
			catch (IOException e)
			{
				throw new RuntimeException(e);
//				log.error("Cannot close reader!");
			}
		}
		return map;
	}

}
