package util;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashMap;
import java.util.Properties;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;

import com.vvk.better.exceptions.LoadingConfDataException;
import com.vvk.better.models.Exchanger;
import com.vvk.better.models.League;
import com.vvk.better.models.LeaguesHolder;
import com.vvk.better.models.exchangers.Betdaq;
import com.vvk.better.models.exchangers.Betfair;
import com.vvk.better.util.CommonConstants;
import com.vvk.better.util.UtilFunctions;

public enum Exchangers implements ExchangerInstance{
	BETDAQ {
		public Exchanger getExchangerInstance(){
			Path path = new Path(BETDAQ_LOG4J_CONFFILE);
			Properties props = new Properties();
			try {
				FileSystem hdfs = FileSystem.get(new Configuration());
				InputStream stream = hdfs.open(path);
				props.load(stream);
			}
			catch (IOException e) {
				e.printStackTrace();
			}
			return new Betdaq(props) {
				@Override
				/**
				 * Loads Configuration data for all defined leagues from a configuration file
				 * 
				 * @throws LoadingConfDataException if a problem occurs during loading
				 */
				public void loadConfiguration() throws LoadingConfDataException
				{
					InputStream confStream = null;
					HashMap<League, URL> leagues = new HashMap<League, URL>();
					String fileName = CommonConstants.RESOURCE_PATH + File.separatorChar + getName().toLowerCase()
							+ conf_file_end;
					BufferedReader reader = null;
					try
					{
						Path path = new Path(fileName);
						FileSystem hdfs = FileSystem.get(new Configuration());
						confStream = hdfs.open(path);
						
						reader = new BufferedReader(new InputStreamReader(confStream));
						String data = reader.readLine();
						while (data != null)
						{
							if (data.startsWith("<" + TAG_NAME))
							{
								boolean isLeagueUsed = true;
								String league = UtilFunctions.extractXmlValue(data, TAG_NAME);
								String country = UtilFunctions.extractXmlValue(data, TAG_COUNTRY);
								League l = null;
								if(league != null && !league.equals("") && country != null && !country.equals(""))
								{
									l = new League(league, country);
//									if(!LeaguesHolder.getLeagues().contains(l)){
//										logger.warn("There is league that is not registered! Please check your configuration files!");
//										isLeagueUsed = false;
//									}
								} else {
									throw new LoadingConfDataException("Wrong league name! Please check your configuration file!");
								}
								if(isLeagueUsed) {
			   					if(data.contains(TAG_QUALNAME)){
			   						String specName = UtilFunctions.extractXmlValue(data, TAG_QUALNAME);
			   						l.setSpecialName(specName);
			   					}
			   					if(data.contains(TAG_QUALCOUNTRY)){
			   						String specCountry = UtilFunctions.extractXmlValue(data, TAG_QUALCOUNTRY);
			   						l.setSpecialCountry(specCountry);
			   					}
			   					String url = UtilFunctions.extractXmlValue(data, TAG_URL);
			   					if (url.equals(""))
			   					{
			   						leagues.put(l, null);
			   					}
			   					else
			   					{
			   						leagues.put(l, new URL(url));
			   					}
								}
							}
							else if (data.startsWith("<" + TAG_FEEKEY))
							{
								String key = UtilFunctions.extractXmlValue(data, TAG_FEEKEY);
								double value = 0;
								if (data.contains(TAG_FEEVALUE))
								{
									String val = UtilFunctions.extractXmlValue(data, TAG_FEEVALUE);
									try
									{
										value = Double.parseDouble(val);
									}
									catch (Exception e)
									{
										throw new LoadingConfDataException("Incorrect fee value!");
									}
								}
								else
								{
									throw new LoadingConfDataException("Wrong fee policy declaration!");
								}
								feePolitics.setFeeProperty(key, value);
							}
							else if (data.startsWith("<" + TAG_MINBET))
							{
								String minBet = UtilFunctions.extractXmlValue(data, TAG_MINBET);
								double bet = 0;
								try
								{
									bet = Double.parseDouble(minBet);
								}
								catch (Exception e)
								{
									throw new LoadingConfDataException("Incorrect fee value!");
								}
								accInfo.setMinBet(bet);
							}
							else if(data.startsWith("<" + TAG_CURRENCY)){
								String currency = UtilFunctions.extractXmlValue(data, TAG_CURRENCY);
								if(currency.length() != 3){
									throw new LoadingConfDataException("Invalid currency!");
								}
								this.currency = currency;
							}
							data = reader.readLine();
						}
						urls = leagues;
					}
					catch (Exception e)
					{
						throw new LoadingConfDataException(e);
					}
					finally
					{
						try
						{
							if (reader != null) reader.close();
							if (confStream != null) confStream.close();
						}
						catch (IOException ie)
						{
							throw new LoadingConfDataException(ie);
						}
					}
				}
			};
		}
	}, 
	BETFAIR{
		public Exchanger getExchangerInstance(){
			Path path = new Path(BETFAIR_LOG4J_CONFFILE);
			Properties props = new Properties();
			try {
				FileSystem hdfs = FileSystem.get(new Configuration());
				InputStream stream = hdfs.open(path);
				props.load(stream);
			}
			catch (IOException e) {
				e.printStackTrace();
			}
			return new Betfair(props){
				@Override
				/**
				 * Loads Configuration data for all defined leagues from a configuration file
				 * 
				 * @throws LoadingConfDataException if a problem occurs during loading
				 */
				public void loadConfiguration() throws LoadingConfDataException
				{
					InputStream confStream = null;
					HashMap<League, URL> leagues = new HashMap<League, URL>();
					String fileName = CommonConstants.RESOURCE_PATH + File.separatorChar + getName().toLowerCase()
							+ conf_file_end;
					BufferedReader reader = null;
					try
					{
						Path path = new Path(fileName);
						FileSystem hdfs = FileSystem.get(new Configuration());
						confStream = hdfs.open(path);
						
						reader = new BufferedReader(new InputStreamReader(confStream));
						String data = reader.readLine();
						while (data != null)
						{
							if (data.startsWith("<" + TAG_NAME))
							{
								boolean isLeagueUsed = true;
								String league = UtilFunctions.extractXmlValue(data, TAG_NAME);
								String country = UtilFunctions.extractXmlValue(data, TAG_COUNTRY);
								League l = null;
								if(league != null && !league.equals("") && country != null && !country.equals(""))
								{
									l = new League(league, country);
//									if(!LeaguesHolder.getLeagues().contains(l)){
//										logger.warn("There is league that is not registered! Please check your configuration files!");
//										isLeagueUsed = false;
//									}
								} else {
									throw new LoadingConfDataException("Wrong league name! Please check your configuration file!");
								}
								if(isLeagueUsed) {
			   					if(data.contains(TAG_QUALNAME)){
			   						String specName = UtilFunctions.extractXmlValue(data, TAG_QUALNAME);
			   						l.setSpecialName(specName);
			   					}
			   					if(data.contains(TAG_QUALCOUNTRY)){
			   						String specCountry = UtilFunctions.extractXmlValue(data, TAG_QUALCOUNTRY);
			   						l.setSpecialCountry(specCountry);
			   					}
			   					String url = UtilFunctions.extractXmlValue(data, TAG_URL);
			   					if (url.equals(""))
			   					{
			   						leagues.put(l, null);
			   					}
			   					else
			   					{
			   						leagues.put(l, new URL(url));
			   					}
								}
							}
							else if (data.startsWith("<" + TAG_FEEKEY))
							{
								String key = UtilFunctions.extractXmlValue(data, TAG_FEEKEY);
								double value = 0;
								if (data.contains(TAG_FEEVALUE))
								{
									String val = UtilFunctions.extractXmlValue(data, TAG_FEEVALUE);
									try
									{
										value = Double.parseDouble(val);
									}
									catch (Exception e)
									{
										throw new LoadingConfDataException("Incorrect fee value!");
									}
								}
								else
								{
									throw new LoadingConfDataException("Wrong fee policy declaration!");
								}
								feePolitics.setFeeProperty(key, value);
							}
							else if (data.startsWith("<" + TAG_MINBET))
							{
								String minBet = UtilFunctions.extractXmlValue(data, TAG_MINBET);
								double bet = 0;
								try
								{
									bet = Double.parseDouble(minBet);
								}
								catch (Exception e)
								{
									throw new LoadingConfDataException("Incorrect fee value!");
								}
								accInfo.setMinBet(bet);
							}
							else if(data.startsWith("<" + TAG_CURRENCY)){
								String currency = UtilFunctions.extractXmlValue(data, TAG_CURRENCY);
								if(currency.length() != 3){
									throw new LoadingConfDataException("Invalid currency!");
								}
								this.currency = currency;
							}
							data = reader.readLine();
						}
						urls = leagues;
					}
					catch (Exception e)
					{
						throw new LoadingConfDataException(e);
					}
					finally
					{
						try
						{
							if (reader != null) reader.close();
							if (confStream != null) confStream.close();
						}
						catch (IOException ie)
						{
							throw new LoadingConfDataException(ie);
						}
					}
				}
			};
		}
	};
	
	private static final String log4j_conffile_end = ".properties";
	public static final String BETDAQ_LOG4J_CONFFILE = CommonConstants.RESOURCE_PATH + File.separatorChar
	+ "betdaq_log4j" + log4j_conffile_end;
	public static final String BETFAIR_LOG4J_CONFFILE = CommonConstants.RESOURCE_PATH + File.separatorChar
	+ "betfair_log4j" + log4j_conffile_end;
}
